////////////////////////////////////////////////////////////////////////////////
//             //                                                             //
//   #####     // Field hospital simulator                                    //
//  ######     // (!) 2013 Giovanni Squillero <giovanni.squillero@polito.it>  //
//  ###   \    //                                                             //
//   ##G  c\   // Field Hospital DAO                                          //
//   #     _\  // Test with MariaDB 10 on win                                 //
//   |   _/    //                                                             //
//   |  _/     //                                                             //
//             // 03FYZ - Tecniche di programmazione 2012-13                  //
////////////////////////////////////////////////////////////////////////////////

package dao;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import bean.Arrival;
import bean.Patient;
import db.DBConnect;

public class FieldHospitalDAO {
	public boolean GetPatient(Patient p) {
		final String sql = "SELECT nome FROM patients WHERE id = ?";

		try {
			Connection conn = DBConnect.getInstance().getConnection();	
			PreparedStatement st = conn.prepareStatement(sql);
			st.setInt(1, p.getId());
			ResultSet rs = st.executeQuery();
			
			boolean ok = false;
			if(rs.next()) {
				p.setName(rs.getString("name"));
				p.setStatus(rs.getString("status"));
				ok = true;
			}
			
			st.close();
			conn.close();
			
			return ok;
			
		} catch (SQLException e) {
			e.printStackTrace();
			throw new RuntimeException(e);
		}
	}

	public Map<Integer, Patient> GetAllPatients() {
		final String sql = "SELECT * FROM patients";
		Map<Integer,Patient> p = new HashMap<Integer,Patient>(); 
		
		try {
			Connection conn = DBConnect.getInstance().getConnection();	
			PreparedStatement st = conn.prepareStatement(sql);
			ResultSet rs = st.executeQuery();
			while(rs.next()) {
				Patient tp = new Patient(rs.getInt("id"), rs.getString("name"));
				p.put(rs.getInt("id"), tp);
			}
			st.close();
			conn.close();
		} catch (SQLException e) {
			e.printStackTrace();
			throw new RuntimeException(e);
		}
		return p;
	}
	
	public List<Arrival> GetAllArrivals() {
		final String sql = "SELECT * FROM arrivals ORDER BY timestamp ASC";
		List<Arrival> l = new ArrayList<Arrival>();

		try {
			Connection conn = DBConnect.getInstance().getConnection();
			PreparedStatement st = conn.prepareStatement(sql);
			ResultSet rs = st.executeQuery();
			while( rs.next() ) {
				l.add(new Arrival(rs.getTimestamp("timestamp"), rs.getInt("patient"), rs.getString("triage")));
			}
			st.close();
			conn.close();		
		} catch (SQLException e) {
			e.printStackTrace();
			throw new RuntimeException(e);
		}		
		return l;
	}
}
