package it.polito;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import simulator.Event;
import simulator.FieldHospital;
import bean.Arrival;
import bean.Patient;
import dao.FieldHospitalDAO;

public class Model {

	FieldHospital hospital = new FieldHospital();
	List<Integer> sfasamentiDottori = new ArrayList<Integer>();
	List<Integer> sfasamentiAssistenti = new ArrayList<Integer>();
	
	public Model(){
		
	}
	
	public void addDottore(String nome, int sfasamento){
		hospital.addDoctor(nome);
		sfasamentiDottori.add(sfasamento);
		System.out.println("dottor "+nome+" aggiunto!");
	}


	public void addAssistente(String nome, int sfasamento){
		hospital.addAssistente(nome);
		sfasamentiAssistenti.add(sfasamento);
		System.out.println("assistente "+nome+" aggiunto!");
	}

	public String startSimulation() {
		hospital.reset();
		String output;
		FieldHospitalDAO dao = new FieldHospitalDAO();

		////////////////////////////////////////////////////////////////////////////
		// Kickstart: read all patients
		Map<Integer, Patient> patientsMap = dao.GetAllPatients();
		
		////////////////////////////////////////////////////////////////////////////
		// Kickstart: parse events
		List<Arrival> l = dao.GetAllArrivals();
		Iterator<Arrival> itr = l.iterator();
		
		long firstTimeArrival = -1;

		while(itr.hasNext()) {
			Arrival a = itr.next();
			long time = a.getTime().getTime();
			
			if(firstTimeArrival == -1) 
				firstTimeArrival = time;
			
			if(time < firstTimeArrival) 
				firstTimeArrival = time;
			
			int patient = a.getPatient();
			patientsMap.get(patient).setStatus("Injured");
			patientsMap.get(patient).setTriage(a.getTriage());
			hospital.addEvent(time, Event.EventEnum.PATIENT_ARRIVAL, patient);
		} 
		
		// finally: set patients data
		hospital.setPatients(patientsMap);

		////////////////////////////////////////////////////////////////////////////
		// Let's roll!
		for(int i=0; i<sfasamentiDottori.size(); i++){
			hospital.addEvent(firstTimeArrival + sfasamentiDottori.get(i)*60*60*1000, Event.EventEnum.DOCTOR_INIZIA_TURNO, i);
		}
		
		for(int i=0; i<sfasamentiAssistenti.size(); i++){
			hospital.addEvent(firstTimeArrival + sfasamentiAssistenti.get(i)*60*60*1000, Event.EventEnum.ASSISTENTE_INIZIA_TURNO, i);
		}
		
		System.out.print("\n\n*** SIMULATION STARTS ***\n\n");
		int max = 0;
		while(hospital.moreEvents() && max < 20000) {
			++max;
			hospital.processNextEvent();
		}

		////////////////////////////////////////////////////////////////////////////
		// Let's roll!
		System.out.print("\n\n*** SIMULATION ENDS ***\n\n");
		Iterator<Patient> itr2 = patientsMap.values().iterator();
		int bodyCount = 0;
		int bodyCountDuringRecoveryGreen = 0;
		int bodyCountDuringRecoveryRed = 0;
		int bodyCountDuringRecoveryYellow = 0;
		int totPatients = 0;
		while(itr2.hasNext()) {
			++totPatients;
			Patient p = itr2.next();
			if(p.getStatusE()==Patient.StatusEnum.DEAD) {
				//System.out.println(p);
				++bodyCount;
			}
			else if (p.getStatusE()==Patient.StatusEnum.DEAD_DURING_RECOVERY_GREEN){
				++bodyCountDuringRecoveryGreen;
			}
			else if (p.getStatusE()==Patient.StatusEnum.DEAD_DURING_RECOVERY_YELLOW){
				++bodyCountDuringRecoveryYellow;
			}
			else if (p.getStatusE()==Patient.StatusEnum.DEAD_DURING_RECOVERY_RED){
				++bodyCountDuringRecoveryRed;
			}
		} 
		
		output = "Death toll: " + (bodyCount+bodyCountDuringRecoveryGreen+bodyCountDuringRecoveryYellow+bodyCountDuringRecoveryRed) + "/" + totPatients;
		output += "\n";
		output += "Death during recovery GREEN: " + bodyCountDuringRecoveryGreen;
		output += "\n";
		output += "Death during recovery YELLOW: " + bodyCountDuringRecoveryYellow;
		output += "\n";
		output += "Death during recovery RED: " + bodyCountDuringRecoveryRed;
		output += "\n";
		output += "Statistics for treated patients:";
		output += "\n";
		for(int t=0; t<4; ++t) {
			output += "* " + hospital.waitTriage[t].getNum() + " " + Patient.TriageEnum.values()[t] + " patients. Waiting time: ";
			output += " " + niceTime(hospital.waitTriage[t].getMin()) + " (min)";
			output += ";";
			output += " " + niceTime(hospital.waitTriage[t].getMax()) + " (max)";
			output += ";";
			output += " " + niceTime(hospital.waitTriage[t].getAverage()) + " (avg)";
			output += "\n";
		}
		
		return output;
	}
	
	static private String niceTime(double time) {
		long min = (long)Math.rint(time / 1000.0 / 60.0);
		long days = min/60/24;
		min -= days *60 *24;
		long hours = min/60;
		min -= hours *60;

		if(days>0) {
			return days + "d " + hours + "h " + min + "'";
		} else if(hours>0) {
			return hours + "h " + min + "'";
		} else {
			return min + "'";
		}
	}
}
